#include "circularplotman.h"

#include <qgridlayout.h>
#include <qucircularplotengine.h>
#include <qucircularplotwidget.h>
#include <quwatcher.h>
#include <cucontext.h>
#include <numberdrawable.h>
#include <QSlider>
#include <QKeyEvent>
#include <QMenu>
#include <QSlider>
#include <QWidgetAction>
#include <qucircularplotdisksector.h>

bool PlotObjFilter::eventFilter(QObject *obj, QEvent *event)
{
    if (event->type() == QEvent::ContextMenu) {
        QMenu *menu = new QMenu(0);
        QAction *ra = menu->addAction("SetReference");
        connect(ra, SIGNAL(toggled(bool)), this, SIGNAL(refEnabled(bool)));
        ra->setCheckable(true);
        ra->setChecked(obj->property("refEnabled").toBool());
        QWidgetAction *sliw = new QWidgetAction(menu);
        QSlider *sli = new QSlider(menu);
        sli->setOrientation(Qt::Horizontal);
        sli->setMinimumSize(QSize(250, 40));
        sliw->setDefaultWidget(sli);

        // slider value[1-300]. span = pow(10, value/100) --> span 0-1000
        // slider with span 20 [-10; 10] --> setValue log10(20) * 100 = 130
        sli->setMinimum(1);
        sli->setMaximum(300);
        sli->setValue(obj->property("yScale").isNull() ? 130 : obj->property("yScale").toInt()); // -10 - 10
        connect(sli, SIGNAL(valueChanged(int)), this, SIGNAL(scaleChange(int)));
        menu->addAction(sliw);
        menu->exec(QCursor::pos());
        delete menu;
        return true;
    } else {
        // standard event processing
        return QObject::eventFilter(obj, event);
    }
};

CircularPlotMan::CircularPlotMan(QWidget *parent)
    : QObject(parent) {
    circular_plots[0] = parent->findChild<QuCircularPlotW *>("plotH");
    circular_plots[1] = parent->findChild<QuCircularPlotW *>("plotV");
}

void CircularPlotMan::setup() {
    for(int i = 0; i < 2; i++) {
        QuCircularPlotEngine *pie = circular_plots[i]->engine();
        pie->a.geom.inner_radius_factor = 0.6;
        pie->a.geom.outer_radius_factor = 0.95;
        pie->a.paint.show_bounds = false;
        pie->a.paint.show_origin = false;
        pie->a.paint.colors.axes = pie->a.paint.colors.background.lighter();
        pie->setYLowerBound(-10);
        pie->setYUpperBound(10);
        pie->setXLowerBound(0);
        circular_plots[i]->setObjectName(i == 0 ? "horplot" : "verplot");
        PlotObjFilter *pf = new PlotObjFilter(circular_plots[i]);
        pf->setObjectName(i == 0 ? "horplot_filter" : "verplot_filter");
        connect(pf, SIGNAL(refEnabled(bool)), this, SLOT(onReferenceEnabled(bool)));
        connect(pf, SIGNAL(scaleChange(int)), this, SLOT(onScaleChange(int)));
        circular_plots[i]->installEventFilter(pf);
        watchers[i] = new QuWatcher(parent());
        CuData o(TTT::RefreshMode, 0x2);
        o[TTT::Period] = 50;
        watchers[i]->getContext()->setOptions(o);
        connect(watchers[i], SIGNAL(newData(CuData)), this, SLOT(update(CuData)));

        watchers[i + 2] = new QuWatcher(parent());
        watchers[i + 2]->getContext()->setOptions(o);
        number_drawables[i] = new NumberDrawable(circular_plots[i]);
        connect(watchers[i + 2], SIGNAL(newData(CuData)),
                number_drawables[i], SLOT(update(CuData)));
        pie->addDrawable(number_drawables[i]);
        QuCircularPlotDiskSector *sec = new QuCircularPlotDiskSector();
        QFont f = circular_plots[i]->font();
        f.setBold(true);
        sec->attributes.paint.font = f;
        pie->addDrawable(sec);
    }
}

void CircularPlotMan::connectSrcs() {
    for(int i = 0; i < 2; i++) {
        CuData o(TTT::RefreshMode, 0x2);
        o[TTT::Period] = 50;
        watchers[i]->getContext()->setOptions(o);
        watchers[i+2]->getContext()->setOptions(o);
    }
    watchers[0]->setSource("$1/CorrHorFast");
    watchers[1]->setSource("$1/CorrVerFast");
    watchers[2]->setSource("$1/CorrHorMaxPerc");
    watchers[3]->setSource("$1/CorrVerMaxPerc");
}

void CircularPlotMan::disconnectSrcs() {
    for(int i = 0; i < 4; i++)
        watchers[i]->unsetSource();
}

void CircularPlotMan::update(const CuData &da) {
    const std::string& src = da.s(TTT::Src);
    int  idx = (src.find("Hor") != std::string::npos) ? 0 : 1;
    QuCircularPlotW *plot = circular_plots[idx];
    const QVector<double>& ref = idx == 0 ? refh : refv;
    if(!da.B(TTT::Err)) {
        const std::vector<double> &v = da.DV(TTT::Value);
        if(circular_plots_x[idx].size() != (int) v.size()) {
            circular_plots_x[idx].resize(v.size());
            for(size_t i = 0;  i < v.size() ; i++)
                circular_plots_x[idx][i] = i;
            plot->engine()->setXUpperBound(v.size());
        }
        QVector<double> y;
        y.resize(v.size());
        for(size_t i = 0; i < v.size(); i++) {
            const double& val = v[i]* 400.0 / 32768.0;
            y[i] = (ref.size() == (int) v.size()) ? (ref[i] -  val) : val; // mA
        }
        plot->setData(idx == 0 ? "hordata" : "verdata", circular_plots_x[idx], y);
    }
    if(da.s(TTT::Type) == "property") { // set low pass filter factor to smooth transitions (just once!)
        QuCircularPlotCurve *c = idx == 0 ? plot->engine()->findCurve("hordata") : plot->engine()->findCurve("verdata");
        c->setLowPassF(0.1);
    }
}

void CircularPlotMan::onReferenceEnabled(bool en) {
    const QString& t = sender()->objectName();
    QPen p;
    p.setWidth(0);
    bool hor = t == "horplot_filter";
    printf("plot is %s\n", qstoc(t));
    QuCircularPlotEngine *e = hor ? circular_plots[0]->engine() : circular_plots[1]->engine();
    const QList<QColor>& palette = e->a.paint.palette.theme_dark ? e->a.paint.palette.dark :
        e->a.paint.palette.light;
    QuCircularPlotCurve *c = hor ? e->findCurve("hordata") : e->findCurve("verdata");
    if(hor && en) {
        refh = c->y_data();
        p.setColor(palette[1]);
    } else if(hor) {
        p.setColor(palette[0]);
        refh.clear();
    } else if(!hor && en) {
        refv = c->y_data();
        p.setColor(palette[1].lighter());
    } else if(!hor) {
        refv.clear();
        p.setColor(palette[0].lighter());
    }
    c->setPen(p);
    emit referenceEnabled(hor, en);
    circular_plots[hor ? 0 : 1]->setProperty("refEnabled", en);
}

void CircularPlotMan::onScaleChange(int v) {
    QString t = sender()->objectName();
    QuCircularPlotW * plot =  circular_plots[( t == "horplot_filter" ) ? 0 : 1];
    plot->setProperty("yScale", v);
    // slider configured from 1 to 10.000
    const double span = pow(10, v/100.0);
    plot->engine()->a.axes.y.lb = -span/2.0;
    plot->engine()->a.axes.y.ub = +span/2.0;
}
