#include "enablefeedbackthread.h"
#include <QVector>
#include <QtDebug>
#include <tango.h>

EnableFeedbackThread::EnableFeedbackThread() : QThread(0), __mode(Check) {
  allEnabled = true;
  allDisabled = true;
  executionOK = true;
  errorMessage = "";
  disabledCorrectors = "";
  pscg = nullptr;
}

EnableFeedbackThread::~EnableFeedbackThread() {
  if (pscg)
    delete pscg;
}

void EnableFeedbackThread::run() {

  // prepare tango group for doing the operation

  try {
    pscg = new Tango::Group("PSC");
    pscg->add("sr/power_supply/psc*");      // H and V planes
    pscg->remove("sr/power_supply/psc*.8"); // remove from group 8-th correctors
    qDebug() << "num psc " << pscg->get_size();
  } catch (Tango::DevFailed &e) {
    errorMessage = "DevFailed during creation of tango group\n";
    executionOK = false;
    return; // it is impossible to go on!
  }
  Tango::GroupReply::enable_exception(true); // anble exception checking one-by-one
  switch (__mode) {

    case EnableFeedbackThread::Enable:
      {
        Tango::DeviceAttribute da("LocalFeedback", true);
        Tango::GroupReplyList rpl = pscg->write_attribute(da);
        if(rpl.has_failed()){
            for (unsigned int r=0; r< rpl.size();r++){
                if (rpl[r].has_failed()){
                    std::cout << rpl[r].dev_name() << " LocalFeedback(true) has failed" << std::endl;
                  }
              }
          }
      }
      break;

    case EnableFeedbackThread::Disable:
      {
        Tango::DeviceAttribute da("LocalFeedback", false);
        Tango::GroupReplyList rpl = pscg->write_attribute(da);
        if(rpl.has_failed()){
            for (unsigned int r=0; r< rpl.size();r++){
                if (rpl[r].has_failed()){
                    std::cout << rpl[r].dev_name() << " LocalFeedback(false) has failed" << std::endl;
                  }
              }
          }

      }
      break;

    case EnableFeedbackThread::Check: {
        //Tango::DevBoolean enabled;
        bool enabled;
        int num_enabled = 0;
        int num_disabled = 0;
        int num_failed = 0;
		allEnabled = false;
		allDisabled = false;
        Tango::GroupAttrReplyList psena = pscg->read_attribute("LocalFeedback");
        for (int i = 0; i < pscg->get_size(); i++) {
            try {
              psena[i] >> enabled;
              if (enabled) {
                  num_enabled++;
                } else {
                  num_disabled++;
                  disabledCorrectors += QString(pscg->get_device_list()[i].c_str()) + "\n";
                }
            } catch (Tango::DevFailed &exc) {
              num_failed++;
              executionOK = false;
              qDebug() << "faild to chek " << pscg->get_device(i)->name().c_str();
            }
          }
        if ((num_enabled == pscg->get_size()) && num_failed == 0) {
            allEnabled = true;
            allDisabled = false;
          }
        if ((num_disabled == pscg->get_size()) && num_failed == 0) {
            allEnabled = false;
            allDisabled = true;
          }
        qDebug() << num_enabled << " " << num_disabled << " " << num_failed ;
        return;
      } break;
    default:
      return;
    }
}
