/***************************************************************************
 *   Copyright (C) 2007 by Vincenzo Forchi`,,,   *
 *   tus@ken   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "gofcontrol.h"
#include "enablefeedbackthread.h"
#include <QtGui>
#include <elettracolors.h>
#include <QMessageBox>
#include <cutango-world.h>
#include <QtDebug>
#include <quwriter.h>

GofControl::GofControl(QWidget *parent, QString dev_name) : QWidget(parent),dname(dev_name)
{
  ui.setupUi(this);

  fbThread = new EnableFeedbackThread();

  QList<unsigned int>  mask;
  mask << 9;
  ui.tLoopStatus->setDisplayMask(mask);
  ui.tLoopStatus->setBooleanDisplay(0, "Feedback Off", "Feedback On", EColor(Elettra::darkYellow), EColor(Elettra::green));
  ui.tLoopStatus->setSource(dname+"/state");

  mask.clear();
  mask << 16 << 17;
  ui.tDispersionStatus->setDisplayMask(mask);
  ui.tDispersionStatus->setBooleanDisplay(0, "Off", "On", EColor(Elettra::darkYellow), EColor(Elettra::green));
  ui.tDispersionStatus->setBooleanDisplay(1, "---", "Error", Qt::white, EColor(Elettra::red));

  /*mask.clear();
        mask << 21 << 22;
        ui.tDriftStatus->setDisplayMask(mask);
        ui.tDriftStatus->setBooleanDisplay(0, "Off", "On", EColor(Elettra::darkYellow), EColor(Elettra::green));
        ui.tDriftStatus->setBooleanDisplay(1, "---", "Error", Qt::white, EColor(Elettra::red));*/

  mask.clear();
  mask << 19 << 20;
  ui.tRecoveryStatus->setDisplayMask(mask);
  ui.tRecoveryStatus->setBooleanDisplay(0, "Off", "On", EColor(Elettra::darkYellow), EColor(Elettra::green));
  ui.tRecoveryStatus->setBooleanDisplay(1, "---", "Error", Qt::white, EColor(Elettra::red));

  ui.tAGCEnabled->setSource(dname+"/agcenabled");
  // ui.tAGCEnabled->setBooleanDisplay("Some Disabled", "All Enabled", EColor(Elettra::darkYellow), EColor(Elettra::green));

  connect(ui.pushTakeReference, SIGNAL(clicked()), this, SLOT(takeReference()));
  connect(ui.pushCheckGof, SIGNAL(clicked()), this, SLOT(checkGof()));
  connect(ui.pushEnableGof, SIGNAL(clicked()), this, SLOT(enableGof()));
  connect(ui.pushDisableGof, SIGNAL(clicked()), this, SLOT(disableGof()));
  connect(ui.tSynchLiberas, SIGNAL(clicked()), this, SLOT(synchLiberas()));

  connect(fbThread, SIGNAL(finished()), this, SLOT(executionFinished()));
}

GofControl::~GofControl()
{
}

void GofControl::synchLiberas()
{
  Tango::GroupCmdReplyList crl;
  bool last_mode;
  QString errors;

  Tango::Group	*ethmon = new Tango::Group("ethmon");
  ethmon->add("'ecgofa*/ethmon/1");

  /* disable ethernet */
  crl = ethmon->command_inout("disableeth");
  last_mode = Tango::GroupReply::enable_exception(true);
  if (crl.has_failed())
    {
      for (unsigned int i = 0; i < crl.size(); i++)
        {
          if (crl[i].has_failed())
            errors.append(QString::fromStdString(crl[i].dev_name())).append(QString("\n"));
        }
      QMessageBox::critical(0, "Error on DisableEth", errors);
      return;
    }
  ::sleep(1);

  /* settime on liberas */
  Tango::Group	*liberas = new Tango::Group("liberas");
  liberas->add("sr/diagnostics/bpm_s*.*");
  liberas->add("sr/diagnostics/lgbpm_s*.*");
  crl = liberas->command_inout("SetTimeOnNextTrigger");
  if (crl.has_failed())
    {
      for (unsigned int i = 0; i < crl.size(); i++)
        {
          if (crl[i].has_failed())
            errors.append(QString::fromStdString(crl[i].dev_name())).append(QString("\n"));
        }
      QMessageBox::critical(0, "Error on SetTimeOnNextTrigger", errors);
      return;
    }

  /* send software event */
  try {
    Tango::DeviceData	in;
    Tango::DevLong		event = 51;
    in << event;
    Tango::DeviceProxy *master = new Tango::DeviceProxy("ecgofmasa5/evg/0");
    master->command_inout("SendSwEvent", in);
  } catch (Tango::DevFailed &e) {
      CuTangoWorld w;
    QMessageBox::critical(0, "Error", w.strerror(e).c_str());
  }

  ::sleep(1);
  /* enable ethernet */
  crl = ethmon->command_inout("enableeth");
  last_mode = Tango::GroupReply::enable_exception(true);
  if (crl.has_failed())
    {
      for (unsigned int i = 0; i < crl.size(); i++)
        {
          if (crl[i].has_failed())
            errors.append(QString::fromStdString(crl[i].dev_name())).append(QString("\n"));
        }
      QMessageBox::critical(0, "Error on DisableEth", errors);
      return;
    }

  Tango::GroupReply::enable_exception(last_mode);
  crl.reset();
}

void GofControl::checkGof()
{
  ui.lineStatusGof->setEnabled(false);
  ui.lineStatusGof->setText("####");
  ui.pushCheckGof->setEnabled(false);
  ui.pushEnableGof->setEnabled(false);
  ui.pushDisableGof->setEnabled(false);
  fbThread->setMode(EnableFeedbackThread::Check);
  fbThread->start();
}

void GofControl::executionFinished()
{
  ui.pushCheckGof->setEnabled(true);
  ui.pushEnableGof->setEnabled(true);
  ui.pushDisableGof->setEnabled(true);
  if (!fbThread->executionOK)
    QMessageBox::warning(0, "RPC Error", QString("Error on\n")+fbThread->errorMessage);
  else if (fbThread->mode() == EnableFeedbackThread::Check)
    {
      ui.lineStatusGof->setEnabled(true);
      if (fbThread->allEnabled)
        {
          ui.lineStatusGof->setText("All Enabled");
          ui.lineStatusGof->setToolTip("");
        }
      else if (fbThread->allDisabled)
        {
          ui.lineStatusGof->setText("All Disabled");
          ui.lineStatusGof->setToolTip("");
        }
      else
        {
          ui.lineStatusGof->setText("Some Disabled");
          ui.lineStatusGof->setToolTip(fbThread->disabledCorrectors);
        }
    }
}

void GofControl::enableGof()
{
  ui.lineStatusGof->setEnabled(false);
  ui.lineStatusGof->setText("####");
  ui.pushCheckGof->setEnabled(false);
  ui.pushEnableGof->setEnabled(false);
  ui.pushDisableGof->setEnabled(false);
  fbThread->setMode(EnableFeedbackThread::Enable);
  fbThread->start();
}

void GofControl::disableGof()
{
  ui.lineStatusGof->setEnabled(false);
  ui.lineStatusGof->setText("####");
  ui.pushCheckGof->setEnabled(false);
  ui.pushEnableGof->setEnabled(false);
  ui.pushDisableGof->setEnabled(false);
  fbThread->setMode(EnableFeedbackThread::Disable);
  fbThread->start();
}

void GofControl::takeReference()
{
  // try {
  //   Tango::DeviceData in;
  //   in << (short) 2;
  //   DeviceFactory::instance()->create(dname)->command_inout("setgofreforbit", in);
  // } catch (DevFailed &e) {

  QuWriter *w = new QuWriter(this);
  w->setTarget(dname + "->setgofreforbit");
  short s{2};
  w->execute(s);
}
