/***************************************************************************
 *   Copyright (C) 2007 by Vincenzo Forchi`,,,   *
 *   tus@ken   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "gofmonitor.h"
#include "qvector_tostd.h"
#include <qwt_symbol.h>
#include <qwt_plot_layout.h>
#include <QFont>
#include <QPainter>
#include <QMessageBox>

#include <elettracolors.h>

#include <QTimer>
#include <QtDebug>
#include <quwatcher.h>


GofMonitor::GofMonitor(QWidget *parent,QString dname) : QWidget(parent),dev_name(dname)
{
    ui.setupUi(this);

    QuWatcher *posw = new QuWatcher(this), *namew = new QuWatcher(this), *srbpmnamew = new QuWatcher(this);
    for (QuWatcher *w : std::list<QuWatcher *>{ posw, namew, srbpmnamew }) {
        w->setSingleShot(true); w->setAutoDestroy(true);
    }

    connect(posw, SIGNAL(newData(CuData)), this, SLOT(onBpmPos(CuData)));
    connect(namew, SIGNAL(newData(CuData)), this, SLOT(onBpmName(CuData)));
    connect(srbpmnamew, SIGNAL(newData(CuData)), this, SLOT(onSrBpmName(CuData)));

    // srbpmnames shalle be read after bpm names is ready. positions and labels belong to the same thread
    // so their results are expected in order
    posw->setObjectName("posw"); namew->setObjectName("namew"); srbpmnamew->setObjectName("srbpmnamew");
    posw->setSource("sr/diagnostics/bpm_s(BpmPosition)");
    namew->setSource("sr/diagnostics/bpm_s(BpmLabel)");


    connect(ui.tabWidget, SIGNAL(currentChanged(int)), this, SLOT(activate(int)));
    connect(ui.comboType, SIGNAL(currentIndexChanged(int)), this, SLOT(changeOrbitSource(int)));
    connect(ui.pushTakeReference, SIGNAL(clicked()), this, SLOT(takeReference()));
    connect(ui.pushClearReference, SIGNAL(clicked()), this, SLOT(clearReference()));

    qRegisterMetaType<QVector<double> >();

    connect(ui.tOrbitHor->meanCurve, SIGNAL(newStatisticsAvailable(const QVector<double>&)), ui.statHorOrbit, SLOT(setValues(const QVector<double>&)), Qt::QueuedConnection);
    connect(ui.tOrbitVer->meanCurve, SIGNAL(newStatisticsAvailable(const QVector<double>&)), ui.statVerOrbit, SLOT(setValues(const QVector<double>&)), Qt::QueuedConnection);

    ui.statHorCorr->precision = 0;
    ui.statVerCorr->precision = 0;
    connect(ui.tCorrHor->meanCurve, SIGNAL(newStatisticsAvailable(const QVector<double>&)), ui.statHorCorr, SLOT(setValues(const QVector<double>&)), Qt::QueuedConnection);
    connect(ui.tCorrVer->meanCurve, SIGNAL(newStatisticsAvailable(const QVector<double>&)), ui.statVerCorr, SLOT(setValues(const QVector<double>&)), Qt::QueuedConnection);

    ui.tOrbitHor->setType(MyTPlot::Orbit);
    ui.tOrbitHor->setAxisTitle(QwtPlot::yLeft, "H [um]");
    ui.tOrbitHor->setAxisTitle(QwtPlot::yRight, "rms H [um]");
    connect(ui.sliderOrbitHor, SIGNAL(valueChanged(int)), ui.tOrbitHor, SLOT(changeMeanScale(int)));
    connect(ui.sliderOrbitHorRms, SIGNAL(valueChanged(int)), ui.tOrbitHor, SLOT(changeRmsScale(int)));
    ui.sliderOrbitHor->setValue(20);
    ui.sliderOrbitHorRms->setValue(6);

    ui.tOrbitVer->setType(MyTPlot::Orbit);

    ui.tOrbitVer->setAxisTitle(QwtPlot::yLeft, "V [um]");
    ui.tOrbitVer->setAxisTitle(QwtPlot::yRight, "rms V [um]");
    connect(ui.sliderOrbitVer, SIGNAL(valueChanged(int)), ui.tOrbitVer, SLOT(changeMeanScale(int)));
    connect(ui.sliderOrbitVerRms, SIGNAL(valueChanged(int)), ui.tOrbitVer, SLOT(changeRmsScale(int)));
    ui.sliderOrbitVer->setValue(20);
    ui.sliderOrbitVerRms->setValue(6);

    std::vector<double> corrPos;
    for (unsigned int i = 0; i < 79; i++)
        corrPos.push_back(i);
    corrPos.push_back(81);
    corrPos.push_back(82);
    corrPos.push_back(83);
    for (int i = 1; i < 13; i++) {
        for (int c = 1; c < 8; c++)
        {
            if ((i == 12) && ((c == 3) || (c == 4)))
                continue;
            corrNames.push_back(QString("S%1.%2").arg(i).arg(c).toStdString());
        }
    }

    ui.tOrbitHor->corr_names = corrNames;
    ui.tOrbitVer->corr_names = (corrNames);
    ui.tCorrHor->corr_names = (corrNames);
    ui.tCorrVer->corr_names = (corrNames);

    ui.tCorrHor->setType(MyTPlot::Corr);
    ui.tCorrHor->setXData(fromStdVector(corrPos));
    ui.tCorrHor->setXAxisAutoscaleEnabled(false);
    ui.tCorrHor->setAxisScale(QwtPlot::xBottom, -1, 84);
    ui.tCorrHor->setAxisTitle(QwtPlot::yLeft, "H [counts]");
    ui.tCorrHor->setAxisTitle(QwtPlot::yRight, "rms H [counts]");
    connect(ui.sliderCorrHor, SIGNAL(valueChanged(int)), ui.tCorrHor, SLOT(changeMeanScale(int)));
    connect(ui.sliderCorrHorRms, SIGNAL(valueChanged(int)), ui.tCorrHor, SLOT(changeRmsScale(int)));
    ui.sliderCorrHor->setValue(20);
    ui.sliderCorrHorRms->setValue(15);

    ui.tCorrVer->setType(MyTPlot::Corr);
    ui.tCorrVer->setXData(fromStdVector(corrPos));
    ui.tCorrVer->setAxisScale(QwtPlot::xBottom, -1, 84);
    ui.tCorrVer->setAxisTitle(QwtPlot::yLeft, "V [counts]");
    ui.tCorrVer->setAxisTitle(QwtPlot::yRight, "rms V [counts]");
    connect(ui.sliderCorrVer, SIGNAL(valueChanged(int)), ui.tCorrVer, SLOT(changeMeanScale(int)));
    connect(ui.sliderCorrVerRms, SIGNAL(valueChanged(int)), ui.tCorrVer, SLOT(changeRmsScale(int)));
    ui.sliderCorrVer->setValue(20);
    ui.sliderCorrVerRms->setValue(15);

    ui.comboType->setCurrentIndex(2);
}

GofMonitor::~GofMonitor()
{
}

void GofMonitor::activate(int tab)
{
    if (tab == 0)
    {
        changeOrbitSource(ui.comboType->currentIndex());
        ui.tOrbitHor->setRmsSource(dev_name+"/rmsorbithor", 100);
        ui.tOrbitVer->setRmsSource(dev_name+"/rmsorbitver", 100);

        ui.tCorrHor->unsetMeanSource();
        ui.tCorrHor->unsetRmsSource();
        ui.tCorrVer->unsetMeanSource();
        ui.tCorrVer->unsetRmsSource();
    }
    else
    {
        ui.tOrbitHor->unsetMeanSource();
        ui.tOrbitHor->unsetRmsSource();
        ui.tOrbitVer->unsetMeanSource();
        ui.tOrbitVer->unsetRmsSource();

        pretty_pri("%s", qstoc(QString(dev_name+"/corrhorfast")));
        ui.tCorrHor->setMeanSource(dev_name+"/corrhorfast", 100);
        ui.tCorrHor->setRmsSource(dev_name+"/rmscorrhor", 100);
        ui.tCorrVer->setMeanSource(dev_name+"/corrverfast", 100);
        ui.tCorrVer->setRmsSource(dev_name+"/rmscorrver", 100);
    }
}

void GofMonitor::changeOrbitSource(int index)
{
    if (index == 0)
    {
        /* absolute */
        clearReference();
        ui.pushTakeReference->setEnabled(false);
        ui.pushClearReference->setEnabled(false);
        ui.tOrbitHor->setMeanSource(dev_name+"/orbithorfast", 100);
        ui.sliderOrbitHor->setValue(20);
        ui.tOrbitVer->setMeanSource(dev_name+"/orbitverfast", 100);
        ui.sliderOrbitVer->setValue(20);
    }
    else if (index == 1)
    {
        /* relative */
        ui.pushTakeReference->setEnabled(true);
        ui.pushClearReference->setEnabled(true);
        ui.tOrbitHor->meanCurve->yReference = fromStdVector(horReference);
        ui.tOrbitHor->setMeanSource(dev_name+"/orbithorfast", 100);
        ui.sliderOrbitHor->setValue(7);
        ui.tOrbitVer->meanCurve->yReference = fromStdVector(verReference);
        ui.tOrbitVer->setMeanSource(dev_name+"/orbitverfast", 100);
        ui.sliderOrbitVer->setValue(7);
    }
    else if (index == 2)
    {
        /* from golden */
        clearReference();
        ui.pushTakeReference->setEnabled(false);
        ui.pushClearReference->setEnabled(false);
        ui.tOrbitHor->setMeanSource(dev_name+"/relativeorbithorfast", 100);
        ui.sliderOrbitHor->setValue(7);
        ui.tOrbitVer->setMeanSource(dev_name+"/relativeorbitverfast", 100);
        ui.sliderOrbitVer->setValue(7);
    }
}

void GofMonitor::onOrbit(const CuData& da) {
    const std::string& s = da.s(TTT::Src);
    bool hor = s.find("orbithor") != std::string::npos;
    hor ? horReference = da.DV(TTT::Value) : verReference = da.DV(TTT::Value);
    if(da.B(TTT::Err) || !da.containsKey(TTT::Value)) {
        perr("%s: %s", __PRETTY_FUNCTION__, da.s(TTT::Message).c_str());
    } else {
        std::vector<double> v = da.DV(TTT::Value);
        hor ? ui.tOrbitHor->meanCurve->yReference = fromStdVector(horReference)
            : ui.tOrbitVer->meanCurve->yReference = fromStdVector(verReference);
    }
}

// first positions arrive, then names (set source order in constructor)
void GofMonitor::onBpmPos(const CuData &da) {
    pretty_pri("%s", datos(da));
    if(!da.B(TTT::Err)) {
        const std::vector<std::string>& pos = da[TTT::Value].toStringVector();
        for(const std::string& p : pos)
            allBpmPositions.push_back(std::stod(p) * 0.001);
    }
    else {
        perr("%s: %s", __PRETTY_FUNCTION__, da.s(TTT::Err).c_str());
    }

}

void GofMonitor::onBpmName(const CuData &da)
{
    pretty_pri("%s", datos(da));
    if(!da.B(TTT::Err)) {
        bpmNames = da[TTT::Value].toStringVector();
        findChild<QuWatcher *>("srbpmnamew")->setSource("$1->getsrbpmnames");
    }
    else {
        perr("%s: %s", __PRETTY_FUNCTION__, da.s(TTT::Err).c_str());
    }
}

void GofMonitor::onSrBpmName(const CuData &da) {
    pretty_pri("%s", datos(da));
    if(!da.B(TTT::Err)) {
        const std::vector<std::string> bpmnames2 = bpmNames; // copy
        bpmNames.clear();
        const std::vector<std::string> &vs = da[TTT::Value].toStringVector();
        for (size_t i = 0; i < vs.size(); i++) {
            std::string s = vs[i];
            s.erase(0, 4);
            bpmNames.push_back(s);
            auto it = std::find(bpmnames2.begin(), bpmnames2.end(), s);
            if(it != bpmnames2.end()) {
                bpmPositions.push_back(allBpmPositions[it - bpmnames2.begin()]);
                pretty_pri("%ld : %s -> %s pos %f", i, vs[i].c_str(), s.c_str(), bpmPositions[bpmPositions.size() -1]);
            }
            else
                pretty_pri("%ld : %s -> `%s` was not found in bpmnames2", i, vs[i].c_str(), s.c_str());
            zeroReference.push_back(0);
        }
        for (int i = 0; i < 5; i++)
            bpmPositions[bpmPositions.size()-1-i] += 259;
    }
    else {
        perr("%s: %s", __PRETTY_FUNCTION__, da.s(TTT::Err).c_str());
    }
    ui.tOrbitHor->bpm_names = bpmNames;
    ui.tOrbitVer->bpm_names = (bpmNames);
    ui.tCorrHor->bpm_names = (bpmNames);
    ui.tCorrVer->bpm_names = (bpmNames);

    ui.tOrbitHor->setXData(fromStdVector(bpmPositions));
    ui.tOrbitHor->setXLowerBound( bpmPositions[0] - 1);
    ui.tOrbitHor->setXUpperBound(bpmPositions[bpmPositions.size()-1] + 1);

    ui.tOrbitVer->setXData(fromStdVector(bpmPositions));
    ui.tOrbitVer->setXLowerBound( bpmPositions[0] - 1);
    ui.tOrbitVer->setXUpperBound(bpmPositions[bpmPositions.size()-1] + 1);

    qDebug() << __PRETTY_FUNCTION__ << bpmPositions;

    activate(0);
}

void GofMonitor::takeReference() {
    QuWatcher *ohw = new QuWatcher(this), *ovw = new QuWatcher(this);
    ohw->setSingleShot(true); ovw->setSingleShot(true);
    ohw->setAutoDestroy(true); ovw->setAutoDestroy(true);
    connect(ohw, SIGNAL(newData(CuData)), this, SLOT(onOrbit(CuData)));
    connect(ovw, SIGNAL(newData(CuData)), this, SLOT(onOrbit(CuData)));
    ohw->setSource(dev_name + "/orbithor");
    ovw->setSource(dev_name + "/orbitver");
}

void GofMonitor::clearReference() {
    ui.tOrbitHor->meanCurve->yReference = fromStdVector(zeroReference);
    ui.tOrbitVer->meanCurve->yReference = fromStdVector(zeroReference);
}


