#include "mytplot.h"
#include "applauncher.h"
#include "gofmonitor.h"
#include <qwt_plot_marker.h>
#include <qwt_plot_picker.h>
#include <qwt_plot_grid.h>
#include <qwt_symbol.h>

#include <QFont>
#include <QPainter>
#include <QMouseEvent>
#include <QMenu>
#include <QLabel>
#include <QTime>
#include <quplotmarkercomponent.h>

#include <elettracolors.h>
#include <quwatcher.h>
#include <cucontext.h>
#include <qwt_scale_map.h>

#include <QContextMenuEvent>
#include <QTimer>
#include <QtDebug>

class MyPlotMarkerFormatter : public QuPlotMarkerFormat_I {
    // QuPlotMarkerFormat_I interface
public:
    QString format(const QuPlotBase *plot, QList<QwtPlotCurve *> curves, const int index, QString &x, QString &y) const {

        if(curves.size() > 0) {
            QString s;
            const MyTPlot *myplot = static_cast<const MyTPlot *>(plot);
            std::string markertext;
            if (myplot->getType() == MyTPlot::Orbit)
                s = QString(": %1um \nrms %2um ").arg(myplot->meanCurve->y(index), 0, 'f', 1).arg(myplot->rmsCurve->y(index), 0, 'f', 2);
            else
                s = QString(": %1 \nrms %2 ").arg(myplot->meanCurve->y(index), 0, 'f', 1).arg(myplot->rmsCurve->y(index), 0, 'f', 2);
            if (myplot->getType() == MyTPlot::Orbit)
                markertext = "bpm_" + myplot->bpm_names[index] +  s.toStdString();
            else
                markertext= "corr_" + myplot->corr_names[index] +  s.toStdString();
            return QString::fromStdString(markertext);
        }
        return QString("%1 %2").arg(x).arg(y);
    }
};

MyBackGround::MyBackGround(QwtPlot *plot)
{
    m_plot = plot;
}

void MyBackGround::draw(QPainter *painter, const QwtScaleMap &xMap, const QwtScaleMap &yMap, const QRectF &) const {
    MyTPlot *mytplot = qobject_cast<MyTPlot*>(m_plot);
    double height = yMap.p1() - yMap.p2();

    QColor c1, c2, c3;
    c1 = EColor(Elettra::greenWater);
    c2 = EColor(Elettra::cyan);
    c3 = EColor(Elettra::blue);

    c1.setAlpha(100);
    c2.setAlpha(100);
    c3.setAlpha(100);

    QFont f = painter->font();
    f.setPointSize(16);
    painter->setFont(f);

    double w = xMap.transform(mytplot->length/12) - xMap.transform(0);
    double off = 0;
    QVector<double> v = mytplot->meanCurve->xData;
    if (v.size())
        off = -5 + xMap.transform(v[0]);
    for (int i = 0; i < 6; i++)
    {
        painter->setPen(c1);
        painter->setBrush(c1);
        QRectF r;
        if (i == 0)
        {
            double w2 = w + off + xMap.transform(mytplot->length/12*2*i) - xMap.transform(0) - xMap.p1();
            r = QRectF(xMap.p1(), yMap.p2(), w2, height);
        }
        else
            r = QRectF(off + xMap.transform(mytplot->length/12*2*i) - xMap.transform(0), yMap.p2(), w, height);
        painter->drawRect(r);
        painter->setPen(c3);
        painter->setBrush(c3);
        painter->drawText(r, Qt::AlignHCenter | Qt::AlignTop, QString("S%1").arg(i*2+1));
    }

    for (int i = 0; i < 6; i++)
    {
        painter->setPen(c2);
        painter->setBrush(c2);
        QRectF r;
        if (i == 5)
        {
            double start = off + xMap.transform(mytplot->length/12*(2*i+1)) - xMap.transform(0);
            r = QRectF(start, yMap.p2(), xMap.p2() - start, height);
        }
        else
            r = QRectF(off + xMap.transform(mytplot->length/12*(2*i+1)) - xMap.transform(0), yMap.p2(), w, height);
        painter->drawRect(r);
        painter->setPen(c3);
        painter->setBrush(c3);
        painter->drawText(r, Qt::AlignHCenter | Qt::AlignTop, QString("S%1").arg(i*2+2));
    }
}

TCurve::TCurve(QObject *parent, const QwtText &title, QColor c, QuPlotCurve *e, bool co)
    : QObject(parent),  QuPlotCurve(title), curveColor(c), yScale(.001), errorCurve(e), compute(co) {
    text = "";
}

TCurve::TCurve(QObject *parent, const QString &title, QColor c, QuPlotCurve *e, bool co)
    : QObject(parent),  QuPlotCurve(title), curveColor(c), yScale(.001), errorCurve(e), compute(co) {
    text = "";
}

void TCurve::computeStatistics()
{
    if(yData.size() <1) return; //C.S. stopgap test - semmem to block nasty bug....
    if (compute_idx++ % 10)
        return;

    MyTPlot *mytplot = qobject_cast<MyTPlot*>(plot());

    double temp = 0;
    mean = 0;
    min = max = yData[0];
    min_idx = max_idx = 0;
    for (unsigned int i = 0; i < yData.size(); i++)
    {
        if (yData[i] < min)
        {
            min_idx = i;
            min = yData[i];
        }
        if (yData[i] > max)
        {
            max_idx = i;
            max = yData[i];
        }
        mean += yData[i];
    }
    mean /= yData.size();
    p2p = max - min;
    for(int i = 0; i < yData.size(); i++)
        temp += (yData[i] - mean) * (yData[i] - mean);

    rms = sqrt(temp / (yData.size()-1));

    std::string unit, minLabel, maxLabel;
    int precision;
    if (mytplot->getType() == MyTPlot::Orbit)
    {
        unit = "um";
        minLabel = mytplot->bpm_names[min_idx];
        maxLabel = mytplot->bpm_names[max_idx];
        precision = 1;
    }
    else
    {
        unit = "";
        minLabel = mytplot->corr_names[min_idx];
        maxLabel = mytplot->corr_names[max_idx];
        precision = 0;
    }

    QVector<double> stat;
    stat << mean << rms << p2p << min << max;

    emit newStatisticsAvailable(stat);
}

void TCurve::refresh(const CuData &var) {
    // pretty_pri("%s", datos(var));
    if(var.B(TTT::Err) || !var.containsKey(TTT::Value)) {
        setPen(QPen(Qt::gray));
        emit newDataAvailable();
        return;
    }
    else {
        QPen p = pen();
        p.setWidth(1);
        p.setColor(curveColor);
        setPen(p);
    }
    const std::vector<double> &v = var.DV(TTT::Value);
    xError.clear();
    yError.clear();
    if (yData.size() != static_cast<int>(v.size()))
        yData.resize(v.size());
    if (xData.size() != yData.size())
        return;
    if (yReference.size() == yData.size())
        for (unsigned i = 0; i < v.size(); i++)
            if (std::isnan(v[i]))
            {
                xError.push_back(xData[i]);
                yError.push_back(yData[i]);
            }
            else
                yData[i] = (v[i] - yReference[i]) * yScale;
    else
        for (unsigned i = 0; i < v.size(); i++)
            if (std::isnan(v[i]))
            {
                xError.push_back(xData[i]);
                yError.push_back(yData[i]);
            }
            else
                yData[i] = v[i] * yScale;
    setData(xData, yData);
    errorCurve->setData(xError, yError);
    if (compute)
        computeStatistics();
    emit newDataAvailable();
}

MyTPlot::MyTPlot(QWidget *parent)
    : QuPlotBase(parent), length(RING_LENGTH),
    closestCurveIdx(-1),
    type(Orbit), menu(0), app(0)
{
    setCanvasBackground(Qt::white);
    setXAxisAutoscaleEnabled(false);
    setYAxisAutoscaleEnabled(false);
    QwtPlotGrid *plotgrid = new QwtPlotGrid;
    plotgrid->enableX(false);
    plotgrid->setPen(QPen(Qt::DotLine));
    plotgrid->attach(this);
    MyBackGround *bg;
    bg = new MyBackGround(this);
    bg->attach(this);
    enableAxis(QwtPlot::yRight);

    QwtSymbol *s, *s1, *s2, *s3;
    QSize	dim1(7,7), dim2(8,8);

    meanErrorCurve = new QuPlotCurve("MeanErrorCurve");
    addCurve("MeanErrorCurve", meanErrorCurve);
    s = new QwtSymbol(QwtSymbol::Ellipse, EColor(Elettra::red), EColor(Elettra::red), dim2 );
    meanErrorCurve->setSymbol(s);
    meanErrorCurve->setYAxis(QwtPlot::yLeft);
    meanErrorCurve->setStyle(QwtPlotCurve::NoCurve);
    meanCurve = new TCurve(this, "Mean", QColor("blue"), meanErrorCurve, true);
    addCurve("Mean", meanCurve);
    s1 = new QwtSymbol(QwtSymbol::Ellipse, EColor(Elettra::blue), EColor(Elettra::blue), dim1 );
    meanCurve->setSymbol(s1);
    meanCurve->setYAxis(QwtPlot::yLeft);
    rmsErrorCurve = new QuPlotCurve("RmsErrorCurve");
    addCurve("RmsErrorCurve", rmsErrorCurve);
    s2 = new QwtSymbol(QwtSymbol::Ellipse, EColor(Elettra::red), EColor(Elettra::red), dim2);
    rmsErrorCurve->setSymbol(s2);
    rmsErrorCurve->setYAxis(QwtPlot::yRight);
    rmsErrorCurve->setStyle(QwtPlotCurve::NoCurve);
    rmsCurve = new TCurve(this, "Rms", QColor("darkGreen"), rmsErrorCurve, true);
    addCurve("Rms", rmsCurve);
    rmsCurve->setStyle(QwtPlotCurve::Sticks);
    s3 = new QwtSymbol(QwtSymbol::Ellipse, EColor(Elettra::darkGreen), EColor(Elettra::darkGreen), dim1);
    rmsCurve->setSymbol(s3);
    rmsCurve->setYAxis(QwtPlot::yRight);
    closestCurve = rmsCurve;

    connect(this, SIGNAL(plotClicked(QPoint, QwtPlotCurve *, double, double, QPoint)),
            this, SLOT(findBpm(QPoint, QwtPlotCurve *, double, double, QPoint)));
    QTimer *updT = new QTimer(this);
    updT->setObjectName("refreshTmr");
    connect(updT, SIGNAL(timeout()), this, SLOT(refresh()));
    updT->setInterval(1000);
    updT->start();

    static_cast<QuPlotMarkerComponent *>(getComponent("marker"))->setFormatter(new MyPlotMarkerFormatter);
}


void MyTPlot::setXData(const QVector<double> &x)
{
    meanCurve->xData = x;
    rmsCurve->xData = x;
}

void MyTPlot::setMeanSource(QString s,  int p) {
    pretty_pri("%s", qstoc(s));
    QuWatcher *w = new QuWatcher(this);
    w->setObjectName("avg_watcher");
    if(p > 0) {
        CuData o(TTT::RefreshMode, 0x2);
        o[TTT::Period] = p;
        w->getContext()->setOptions(o);
    }
    connect(w, SIGNAL(newData(CuData)), meanCurve, SLOT(refresh(CuData)));
    w->setSource(s);
    QTimer *t = findChild<QTimer *>("refreshTmr");
    if(p > 0 && t->interval() > p)
        t->setInterval(p);
}

void MyTPlot::setRmsSource(QString s, int period)
{
    pretty_pri("%s", qstoc(s));
    QuWatcher *w = new QuWatcher(this);
    w->setObjectName("rms_watcher");
    if(period > 0) {
        CuData o(TTT::RefreshMode, 0x2);
        o[TTT::Period] = period;
        w->getContext()->setOptions(o);
    }
    connect(w, SIGNAL(newData(CuData)), rmsCurve, SLOT(refresh(CuData)));
    w->setSource(s);
    QTimer *t = findChild<QTimer *>("refreshTmr");
    if(period > 0 && t->interval() > period)
        t->setInterval(period);
}

void MyTPlot::unsetMeanSource() {
    QuWatcher *w = findChild<QuWatcher *>("avg_watcher");
    if(w) {
        pretty_pri("deleting watcher for %s", qstoc(w->source()));
        delete w;
    }
    else pretty_pri("watcher not found");
}

void MyTPlot::unsetRmsSource() {
    QuWatcher *w = findChild<QuWatcher *>("rms_watcher");
    if(w) {
        pretty_pri("deleting watcher for %s", qstoc(w->source()));
        delete w;
    } else pretty_pri("watcher not found");
}

void MyTPlot::changeMeanScale(int v)
{
    int s = (int) pow(10, v*.2);
    setYUpperBound(s);
    setYLowerBound(-s);
    refresh();
}

void MyTPlot::changeRmsScale(int v)
{
    int s = (int) pow(10, v*.2);
    setAxisScale(QwtPlot::yRight, 0, s);
    refresh();
}

void MyTPlot::setType(Type t)
{
    type = t;
    if (type == Orbit) {
        length = RING_LENGTH;
        meanCurve->yScale = .001;
        rmsCurve->yScale = .001;
    }
    else {
        length = 84;
        meanCurve->yScale = 1;
        meanCurve->setStyle(QwtPlotCurve::Sticks);
        rmsCurve->yScale = 1;
    }
}

MyTPlot::Type MyTPlot::getType() const {
    return type;
}

void MyTPlot::findBpm(const QPoint& pos, bool showMenu, bool launch) {
    QString s;
    std::string markertext, helpera;
    QwtPlotCurve *c;
    int closest = QuPlotBase::closestCurveIdx(pos, &c);
    if(closest > 0 && c) {
        std::string dev_name;
        if (type == Orbit)  {
            dev_name = "sr/diagnostics/bpm_" + bpm_names[closest];
            std::string appName = "bpm";
            helpera = appName + " " + dev_name;
        }
        else {
            dev_name = "sr/feedback/corr_" + bpm_names[closest];
            helpera = "corr " + dev_name;
        }
        if(showMenu) {
            if(menu)
                delete menu;
            menu = new QMenu(this);
            QString s;
            if (type == Orbit)
                s = "Launch BPM_" + QString::fromStdString(bpm_names[closest]);
            else
                s = "Launch CORR_" + QString::fromStdString(bpm_names[closest]);
            QAction *a = menu->addAction(s, this, SLOT(launchHelperApplication()));
            a->setProperty("app", helpera.c_str());
            menu->popup(QCursor::pos());
        } else if (launch)
            launchHelperApplication(helpera);
    }
}

void MyTPlot::launchHelperApplication(const std::string& cmd)
{
    std::string c = cmd.empty() ? qobject_cast<QAction *>(sender())->property("app").toString().toStdString() : cmd;
    if (app)
        delete app;
    app = new AppLauncher();
    pretty_pri("starting `%s`", c.c_str());
    app->start(c.c_str());
}

//---------------------------------------------------------------------
QString MyTPlot::markerText(const QwtPlotCurve *, const int index) {

}

void MyTPlot::contextMenuEvent(QContextMenuEvent *e) {
    findBpm(canvas()->mapFromParent(e->pos()), true, false);
}

void MyTPlot::mouseDoubleClickEvent(QMouseEvent *e) {
    findBpm(canvas()->mapFromParent(e->pos()), false, true);
}
