#ifndef _MYTPLOT_H
#define _MYTPLOT_H

#define RING_LENGTH 259.2

#include <QObject>
#include <qwt_text_engine.h>
#include <qwt_plot_item.h>
#include <quplotcurve.h>
#include <cudatalistener.h>
#include <quplot_base.h>

class QMenu;
class EApplicationLauncher;
class QLabel;
class AppLauncher;

class TCurve : public QObject, public QuPlotCurve
{
    Q_OBJECT
public:
    TCurve(QObject *, const QwtText &, QColor, QuPlotCurve*, bool = false);
    TCurve(QObject *, const QString &, QColor, QuPlotCurve*, bool = false);

    void computeStatistics();

signals:
    void newDataAvailable();
    void newStatisticsAvailable(const QVector<double> &);

protected slots:
    void refresh(const CuData& data);

public:
    QVector<double>	xData, yData, yReference;
    QVector<double>	xError, yError;
    QColor	curveColor;
    double	yScale;
    QuPlotCurve	*errorCurve;
    bool	compute;
    double	min, max, mean, rms, p2p;
    int	min_idx, max_idx;
    int	compute_idx;
    QString	text;
};

class MyBackGround: public QwtPlotItem
{
public:
    MyBackGround(QwtPlot *parent);
    void draw(QPainter *, const QwtScaleMap &, const QwtScaleMap &, const QRectF &) const;
    std::vector<double> bpmPositions;
private:
    QwtPlot *m_plot;
};

class MyTPlot : public QuPlotBase
{
    friend class MyBackGround;
    Q_OBJECT
public:
    enum Type {
        Orbit = 0,
        Corr
    };

    MyTPlot(QWidget*);

    void setXData(const QVector<double> &x);

    // QTango RefreshMode = AUTO_REFRESH is mapped into 0x02, CuTreader::ChangeEventRefresh
    void setMeanSource(QString,int period);
    void setRmsSource(QString, int period);

    void unsetMeanSource();
    void unsetRmsSource();

    void setType(Type);
    Type getType() const;

    double length;

public slots:
    void changeMeanScale(int);
    void changeRmsScale(int);

protected slots:
    void findBpm(const QPoint& pt, bool showMenu, bool launch);
    void launchHelperApplication(const std::string &cmd = std::string());

public:
    TCurve*	meanCurve;
    TCurve*	rmsCurve;
    std::vector<std::string> bpm_names, corr_names;

protected:
    //void keyPressEvent(QKeyEvent *);

    QuPlotCurve*	meanErrorCurve;
    QuPlotCurve*	rmsErrorCurve;

    int		closestCurveIdx;
    QuPlotCurve*	closestCurve;

    Type	type;

    QMenu*	menu;
    AppLauncher *app;
    virtual QString markerText(const QwtPlotCurve *relatedCurve, const int index);
    QString markertext;

    void contextMenuEvent(QContextMenuEvent *e);
    void mouseDoubleClickEvent(QMouseEvent *event);
};

#endif
