#include "numberdrawable.h"
#include <QPainter>
#include <QWidget>
#include <cudata.h>
#include <qupalette.h>

NumberDrawable::NumberDrawable(QObject *parent)
    : QObject(parent),  m_s("#") {}

void NumberDrawable::update(const CuData &da) {
    const std::string& src = da.s(TTT::Src);
    if(!da.B(TTT::Err)) {
        m_s = da.s(TTT::Value);
    } else {
        m_s = "##";
    }
}

bool NumberDrawable::scales() {
    return true;
}

int NumberDrawable::z() const {
    return 1;
}

void NumberDrawable::draw(QPainter *p,
                          const QuCircularPlotEngine */*plot_e*/,
                          double /*inner_radius*/,
                          double /*outer_radius*/,
                          const QRectF &rect,
                          QWidget */*widget*/) {
    QFont f = p->font();
    f.setPointSizeF(f.pointSizeF() * 1.4);
    p->setFont(f);
    const QString& txt(m_s.c_str());
    double w = p->fontMetrics().horizontalAdvance(m_s.c_str());
    double h = p->fontMetrics().height();
    QPointF c( rect.center().x() - w/2.0, rect.center().y() + h/2);
    QColor bg = QuPalette::instance()->value(QString("default"));
    // QBrush b(bg/*.lighter()*/);
    // p->setBrush(b);
    p->setPen(bg.darker());
    p->drawRoundedRect(c.x() - 4, c.y() - h + 6, w + 8, h + 6, 4.0, 4.0);
    p->setPen(QuPalette::instance()->text(QString("default")).lighter());
    p->drawText(c, m_s.c_str());
}

